@@ -2,8 +2,10 @@ from IoTuring.ClassManager.ClassManager import ClassManager
 from IoTuring.ClassManager import consts
 from IoTuring.Warehouse.Warehouse import Warehouse
 
-class WarehouseClassManager(ClassManager): # Class to load Entities from the Entitties dir and get them from name 
+
+# Class to load Entities from the Entitties dir and get them from name
+class WarehouseClassManager(ClassManager):
     def __init__(self):
         ClassManager.__init__(self)
         self.baseClass = Warehouse
-        self.GetModulesFilename(consts.WAREHOUSES_PATH) 
\ No newline at end of file
+        self.GetModulesFilename(consts.WAREHOUSES_PATH)
