@@ -11,6 +11,10 @@ LOG_DEBUG = 4
 LOG_DEVELOPMENT = 5
 
 
+# On/off states as strings:
+STATE_ON = "ON"
+STATE_OFF = "OFF"
+
 # Fill start of string with spaces to jusitfy the message (0: no padding)
 # First for type, second for source
 STRINGS_LENGTH = [8, 30]
@@ -18,9 +22,9 @@ STRINGS_LENGTH = [8, 30]
 # number of spaces to separe the message from the previuos part of the row
 PRESTRING_MESSAGE_SEPARATOR_LEN = 2
 # before those spaces I add this string
-LONG_MESSAGE_PRESTRING_CHAR = ' ' 
+LONG_MESSAGE_PRESTRING_CHAR = ' '
 
 CONSOLE_LOG_LEVEL = LOG_INFO
 FILE_LOG_LEVEL = LOG_WARNING
 
-MESSAGE_WIDTH = 95
\ No newline at end of file
+MESSAGE_WIDTH = 95
