@@ -1,6 +1,7 @@
 from IoTuring.Logger.Logger import Logger
 from IoTuring.Warehouse.Warehouse import Warehouse
 
+
 class ConsoleWarehouse(Warehouse):
     NAME = "Console"
 
@@ -8,6 +9,5 @@ class ConsoleWarehouse(Warehouse):
         for entity in self.GetEntities():
             for entitySensor in entity.GetEntitySensors():
                 if(entitySensor.HasValue()):
-                    self.Log(Logger.LOG_MESSAGE,entitySensor.GetId() + ": " + entitySensor.GetValue())
-
-    
\ No newline at end of file
+                    self.Log(Logger.LOG_MESSAGE, entitySensor.GetId() +
+                             ": " + entitySensor.GetValue())
